@@ -66,7 +66,7 @@ static ULONG STDMETHODCALLTYPE d3d11_blend_state_AddRef(ID3D11BlendState *iface)
 
     if (refcount == 1)
     {
-        ID3D11Device_AddRef(state->device);
+        ID3D11Device1_AddRef(state->device);
         wined3d_mutex_lock();
         wined3d_blend_state_incref(state->wined3d_state);
         wined3d_mutex_unlock();
@@ -84,13 +84,13 @@ static ULONG STDMETHODCALLTYPE d3d11_blend_state_Release(ID3D11BlendState *iface
 
     if (!refcount)
     {
-        ID3D11Device *device = state->device;
+        ID3D11Device1 *device = state->device;
 
         wined3d_mutex_lock();
         wined3d_blend_state_decref(state->wined3d_state);
         wined3d_mutex_unlock();
 
-        ID3D11Device_Release(device);
+        ID3D11Device1_Release(device);
     }
 
     return refcount;
@@ -103,7 +103,7 @@ static void STDMETHODCALLTYPE d3d11_blend_state_GetDevice(ID3D11BlendState *ifac
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    *device = state->device;
+    *device = (ID3D11Device *)state->device;
     ID3D11Device_AddRef(*device);
 }
 
@@ -206,7 +206,7 @@ static void STDMETHODCALLTYPE d3d10_blend_state_GetDevice(ID3D10BlendState1 *ifa
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    ID3D11Device_QueryInterface(state->device, &IID_ID3D10Device, (void **)device);
+    ID3D11Device1_QueryInterface(state->device, &IID_ID3D10Device, (void **)device);
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_blend_state_GetPrivateData(ID3D10BlendState1 *iface,
@@ -294,7 +294,7 @@ static const struct ID3D10BlendState1Vtbl d3d10_blend_state_vtbl =
 static void STDMETHODCALLTYPE d3d_blend_state_wined3d_object_destroyed(void *parent)
 {
     struct d3d_blend_state *state = parent;
-    struct d3d_device *device = impl_from_ID3D11Device(state->device);
+    struct d3d_device *device = impl_from_ID3D11Device1(state->device);
 
     wine_rb_remove(&device->blend_states, &state->entry);
     wined3d_private_store_cleanup(&state->private_store);
@@ -395,7 +395,7 @@ HRESULT d3d_blend_state_create(struct d3d_device *device, const D3D11_BLEND_DESC
     }
     wined3d_mutex_unlock();
 
-    ID3D11Device_AddRef(object->device = &device->ID3D11Device_iface);
+    ID3D11Device1_AddRef(object->device = &device->ID3D11Device1_iface);
 
     TRACE("Created blend state %p.\n", object);
     *state = object;
@@ -466,7 +466,7 @@ static ULONG STDMETHODCALLTYPE d3d11_depthstencil_state_AddRef(ID3D11DepthStenci
 static void d3d_depthstencil_state_cleanup(struct d3d_depthstencil_state *state)
 {
     wined3d_private_store_cleanup(&state->private_store);
-    ID3D11Device_Release(state->device);
+    ID3D11Device1_Release(state->device);
 }
 
 static ULONG STDMETHODCALLTYPE d3d11_depthstencil_state_Release(ID3D11DepthStencilState *iface)
@@ -478,7 +478,7 @@ static ULONG STDMETHODCALLTYPE d3d11_depthstencil_state_Release(ID3D11DepthStenc
 
     if (!refcount)
     {
-        struct d3d_device *device = impl_from_ID3D11Device(state->device);
+        struct d3d_device *device = impl_from_ID3D11Device1(state->device);
         wined3d_mutex_lock();
         wine_rb_remove(&device->depthstencil_states, &state->entry);
         d3d_depthstencil_state_cleanup(state);
@@ -496,7 +496,7 @@ static void STDMETHODCALLTYPE d3d11_depthstencil_state_GetDevice(ID3D11DepthSten
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    *device = state->device;
+    *device = (ID3D11Device *)state->device;
     ID3D11Device_AddRef(*device);
 }
 
@@ -600,7 +600,7 @@ static void STDMETHODCALLTYPE d3d10_depthstencil_state_GetDevice(ID3D10DepthSten
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    ID3D11Device_QueryInterface(state->device, &IID_ID3D10Device, (void **)device);
+    ID3D11Device1_QueryInterface(state->device, &IID_ID3D10Device, (void **)device);
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_depthstencil_state_GetPrivateData(ID3D10DepthStencilState *iface,
@@ -671,8 +671,8 @@ static HRESULT d3d_depthstencil_state_init(struct d3d_depthstencil_state *state,
     wined3d_private_store_init(&state->private_store);
     state->desc = *desc;
 
-    state->device = &device->ID3D11Device_iface;
-    ID3D11Device_AddRef(state->device);
+    state->device = &device->ID3D11Device1_iface;
+    ID3D11Device1_AddRef(state->device);
 
     return S_OK;
 }
@@ -831,7 +831,7 @@ static ULONG STDMETHODCALLTYPE d3d11_rasterizer_state_AddRef(ID3D11RasterizerSta
 
     if (refcount == 1)
     {
-        ID3D11Device_AddRef(state->device);
+        ID3D11Device1_AddRef(state->device);
         wined3d_mutex_lock();
         wined3d_rasterizer_state_incref(state->wined3d_state);
         wined3d_mutex_unlock();
@@ -849,13 +849,13 @@ static ULONG STDMETHODCALLTYPE d3d11_rasterizer_state_Release(ID3D11RasterizerSt
 
     if (!refcount)
     {
-        ID3D11Device *device = state->device;
+        ID3D11Device1 *device = state->device;
 
         wined3d_mutex_lock();
         wined3d_rasterizer_state_decref(state->wined3d_state);
         wined3d_mutex_unlock();
 
-        ID3D11Device_Release(device);
+        ID3D11Device1_Release(device);
     }
 
     return refcount;
@@ -868,7 +868,7 @@ static void STDMETHODCALLTYPE d3d11_rasterizer_state_GetDevice(ID3D11RasterizerS
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    *device = state->device;
+    *device = (ID3D11Device *)state->device;
     ID3D11Device_AddRef(*device);
 }
 
@@ -972,7 +972,7 @@ static void STDMETHODCALLTYPE d3d10_rasterizer_state_GetDevice(ID3D10RasterizerS
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    ID3D11Device_QueryInterface(state->device, &IID_ID3D10Device, (void **)device);
+    ID3D11Device1_QueryInterface(state->device, &IID_ID3D10Device, (void **)device);
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_rasterizer_state_GetPrivateData(ID3D10RasterizerState *iface,
@@ -1037,7 +1037,7 @@ static const struct ID3D10RasterizerStateVtbl d3d10_rasterizer_state_vtbl =
 static void STDMETHODCALLTYPE d3d_rasterizer_state_wined3d_object_destroyed(void *parent)
 {
     struct d3d_rasterizer_state *state = parent;
-    struct d3d_device *device = impl_from_ID3D11Device(state->device);
+    struct d3d_device *device = impl_from_ID3D11Device1(state->device);
 
     wine_rb_remove(&device->rasterizer_states, &state->entry);
     wined3d_private_store_cleanup(&state->private_store);
@@ -1081,7 +1081,7 @@ static HRESULT d3d_rasterizer_state_init(struct d3d_rasterizer_state *state, str
         return hr;
     }
 
-    ID3D11Device_AddRef(state->device = &device->ID3D11Device_iface);
+    ID3D11Device1_AddRef(state->device = &device->ID3D11Device1_iface);
 
     return S_OK;
 }
@@ -1194,7 +1194,7 @@ static ULONG STDMETHODCALLTYPE d3d11_sampler_state_AddRef(ID3D11SamplerState *if
 
     if (refcount == 1)
     {
-        ID3D11Device_AddRef(state->device);
+        ID3D11Device1_AddRef(state->device);
         wined3d_mutex_lock();
         wined3d_sampler_incref(state->wined3d_sampler);
         wined3d_mutex_unlock();
@@ -1212,13 +1212,13 @@ static ULONG STDMETHODCALLTYPE d3d11_sampler_state_Release(ID3D11SamplerState *i
 
     if (!refcount)
     {
-        ID3D11Device *device = state->device;
+        ID3D11Device1 *device = state->device;
 
         wined3d_mutex_lock();
         wined3d_sampler_decref(state->wined3d_sampler);
         wined3d_mutex_unlock();
 
-        ID3D11Device_Release(device);
+        ID3D11Device1_Release(device);
     }
 
     return refcount;
@@ -1231,7 +1231,7 @@ static void STDMETHODCALLTYPE d3d11_sampler_state_GetDevice(ID3D11SamplerState *
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    *device = state->device;
+    *device = (ID3D11Device *)state->device;
     ID3D11Device_AddRef(*device);
 }
 
@@ -1335,7 +1335,7 @@ static void STDMETHODCALLTYPE d3d10_sampler_state_GetDevice(ID3D10SamplerState *
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    ID3D11Device_QueryInterface(state->device, &IID_ID3D10Device, (void **)device);
+    ID3D11Device1_QueryInterface(state->device, &IID_ID3D10Device, (void **)device);
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_sampler_state_GetPrivateData(ID3D10SamplerState *iface,
@@ -1400,7 +1400,7 @@ static const struct ID3D10SamplerStateVtbl d3d10_sampler_state_vtbl =
 static void STDMETHODCALLTYPE d3d_sampler_wined3d_object_destroyed(void *parent)
 {
     struct d3d_sampler_state *state = parent;
-    struct d3d_device *device = impl_from_ID3D11Device(state->device);
+    struct d3d_device *device = impl_from_ID3D11Device1(state->device);
 
     wine_rb_remove(&device->sampler_states, &state->entry);
     wined3d_private_store_cleanup(&state->private_store);
@@ -1494,8 +1494,8 @@ static HRESULT d3d_sampler_state_init(struct d3d_sampler_state *state, struct d3
         return hr;
     }
 
-    state->device = &device->ID3D11Device_iface;
-    ID3D11Device_AddRef(state->device);
+    state->device = &device->ID3D11Device1_iface;
+    ID3D11Device1_AddRef(state->device);
 
     return S_OK;
 }
