@@ -78,7 +78,7 @@ static ULONG STDMETHODCALLTYPE d3d11_texture1d_AddRef(ID3D11Texture1D *iface)
 
     if (refcount == 1)
     {
-        ID3D11Device_AddRef(texture->device);
+        ID3D11Device1_AddRef(texture->device);
         wined3d_mutex_lock();
         wined3d_texture_incref(texture->wined3d_texture);
         wined3d_mutex_unlock();
@@ -96,14 +96,14 @@ static ULONG STDMETHODCALLTYPE d3d11_texture1d_Release(ID3D11Texture1D *iface)
 
     if (!refcount)
     {
-        ID3D11Device *device = texture->device;
+        ID3D11Device1 *device = texture->device;
 
         wined3d_mutex_lock();
         wined3d_texture_decref(texture->wined3d_texture);
         wined3d_mutex_unlock();
         /* Release the device last, it may cause the wined3d device to be
          * destroyed. */
-        ID3D11Device_Release(device);
+        ID3D11Device1_Release(device);
     }
 
     return refcount;
@@ -115,7 +115,7 @@ static void STDMETHODCALLTYPE d3d11_texture1d_GetDevice(ID3D11Texture1D *iface,
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    *device = texture->device;
+    *device = (ID3D11Device *)texture->device;
     ID3D11Device_AddRef(*device);
 }
 
@@ -287,7 +287,7 @@ static void STDMETHODCALLTYPE d3d10_texture1d_GetDevice(ID3D10Texture1D *iface,
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    ID3D11Device_QueryInterface(texture->device, &IID_ID3D10Device, (void **)device);
+    ID3D11Device1_QueryInterface(texture->device, &IID_ID3D10Device, (void **)device);
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_texture1d_GetPrivateData(ID3D10Texture1D *iface,
@@ -510,8 +510,8 @@ HRESULT d3d_texture1d_create(struct d3d_device *device, const D3D11_TEXTURE1D_DE
     }
     wined3d_mutex_unlock();
 
-    texture->device = &device->ID3D11Device_iface;
-    ID3D11Device_AddRef(texture->device);
+    texture->device = &device->ID3D11Device1_iface;
+    ID3D11Device1_AddRef(texture->device);
 
     TRACE("Created texture %p.\n", texture);
     *out = texture;
@@ -566,7 +566,7 @@ static ULONG STDMETHODCALLTYPE d3d11_texture2d_AddRef(ID3D11Texture2D *iface)
 
     if (refcount == 1)
     {
-        ID3D11Device_AddRef(texture->device);
+        ID3D11Device1_AddRef(texture->device);
         wined3d_mutex_lock();
         wined3d_texture_incref(texture->wined3d_texture);
         wined3d_mutex_unlock();
@@ -584,14 +584,14 @@ static ULONG STDMETHODCALLTYPE d3d11_texture2d_Release(ID3D11Texture2D *iface)
 
     if (!refcount)
     {
-        ID3D11Device *device = texture->device;
+        ID3D11Device1 *device = texture->device;
 
         wined3d_mutex_lock();
         wined3d_texture_decref(texture->wined3d_texture);
         wined3d_mutex_unlock();
         /* Release the device last, it may cause the wined3d device to be
          * destroyed. */
-        ID3D11Device_Release(device);
+        ID3D11Device1_Release(device);
     }
 
     return refcount;
@@ -603,7 +603,7 @@ static void STDMETHODCALLTYPE d3d11_texture2d_GetDevice(ID3D11Texture2D *iface,
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    *device = texture->device;
+    *device = (ID3D11Device *)texture->device;
     ID3D11Device_AddRef(*device);
 }
 
@@ -789,7 +789,7 @@ static void STDMETHODCALLTYPE d3d10_texture2d_GetDevice(ID3D10Texture2D *iface,
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    ID3D11Device_QueryInterface(texture->device, &IID_ID3D10Device, (void **)device);
+    ID3D11Device1_QueryInterface(texture->device, &IID_ID3D10Device, (void **)device);
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_texture2d_GetPrivateData(ID3D10Texture2D *iface,
@@ -1066,7 +1066,7 @@ HRESULT d3d_texture2d_create(struct d3d_device *device, const D3D11_TEXTURE2D_DE
     }
     wined3d_mutex_unlock();
 
-    ID3D11Device_AddRef(texture->device = &device->ID3D11Device_iface);
+    ID3D11Device1_AddRef(texture->device = &device->ID3D11Device1_iface);
 
     TRACE("Created texture %p.\n", texture);
     *out = texture;
@@ -1120,7 +1120,7 @@ static ULONG STDMETHODCALLTYPE d3d11_texture3d_AddRef(ID3D11Texture3D *iface)
 
     if (refcount == 1)
     {
-        ID3D11Device_AddRef(texture->device);
+        ID3D11Device1_AddRef(texture->device);
         wined3d_mutex_lock();
         wined3d_texture_incref(texture->wined3d_texture);
         wined3d_mutex_unlock();
@@ -1146,14 +1146,14 @@ static ULONG STDMETHODCALLTYPE d3d11_texture3d_Release(ID3D11Texture3D *iface)
 
     if (!refcount)
     {
-        ID3D11Device *device = texture->device;
+        ID3D11Device1 *device = texture->device;
 
         wined3d_mutex_lock();
         wined3d_texture_decref(texture->wined3d_texture);
         wined3d_mutex_unlock();
         /* Release the device last, it may cause the wined3d device to be
          * destroyed. */
-        ID3D11Device_Release(device);
+        ID3D11Device1_Release(device);
     }
 
     return refcount;
@@ -1165,7 +1165,7 @@ static void STDMETHODCALLTYPE d3d11_texture3d_GetDevice(ID3D11Texture3D *iface,
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    *device = texture->device;
+    *device = (ID3D11Device *)texture->device;
     ID3D11Device_AddRef(*device);
 }
 
@@ -1287,7 +1287,7 @@ static void STDMETHODCALLTYPE d3d10_texture3d_GetDevice(ID3D10Texture3D *iface,
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    ID3D11Device_QueryInterface(texture->device, &IID_ID3D10Device, (void **)device);
+    ID3D11Device1_QueryInterface(texture->device, &IID_ID3D10Device, (void **)device);
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_texture3d_GetPrivateData(ID3D10Texture3D *iface,
@@ -1495,8 +1495,8 @@ static HRESULT d3d_texture3d_init(struct d3d_texture3d *texture, struct d3d_devi
     wined3d_mutex_unlock();
     texture->desc.MipLevels = levels;
 
-    texture->device = &device->ID3D11Device_iface;
-    ID3D11Device_AddRef(texture->device);
+    texture->device = &device->ID3D11Device1_iface;
+    ID3D11Device1_AddRef(texture->device);
 
     return S_OK;
 }
