@@ -417,7 +417,8 @@ static void test_persist_save_data(const char *testname, IHlink *lnk,
                 (expected_data_alt == expected_hlink_data3_ie7 && i == 109) /* Win10 */ ||
                 (expected_data_alt == expected_hlink_data5_ie7 && i == 107) /* Win10 */)
             {
-                ok(data[i] == 0 || broken(data[i] == 1), "Expected 0 or 1, got %d\n", data[i]);
+                ok(data[i] == 0 || broken(data[i] == 1) || broken(data[i] == 3),
+                   "Expected 0 or 1, got %d\n", data[i]);
                 continue;
             }
             if ((expected_data_alt[i] != data[i]) &&
