@@ -140,6 +140,11 @@ function test_javascript() {
     test_exposed("toISOString", Date.prototype, v >= 9);
     test_exposed("isArray", Array, v >= 9);
     test_exposed("indexOf", Array.prototype, v >= 9);
+    /* FIXME: IE8 implements weird semi-functional property descriptors. */
+    if(v != 8) {
+        test_exposed("getOwnPropertyDescriptor", Object, v >= 8);
+        test_exposed("defineProperty", Object, v >= 8);
+    }
 
     test_parses("if(false) { o.default; }", v >= 9);
     test_parses("if(false) { o.with; }", v >= 9);
