@@ -29,7 +29,6 @@
 #include "winnls.h"
 #include "shlwapi.h"
 #include "wingdi.h"
-#include "wine/debug.h"
 #include "msi.h"
 #include "msiquery.h"
 #include "objidl.h"
@@ -39,13 +38,16 @@
 #include "winver.h"
 #include "urlmon.h"
 #include "shlobj.h"
-#include "wine/unicode.h"
 #include "objbase.h"
 #include "msidefs.h"
 #include "sddl.h"
 
+#include "wine/heap.h"
+#include "wine/debug.h"
+#include "wine/unicode.h"
+
 #include "msipriv.h"
-#include "msiserver.h"
+#include "winemsi.h"
 #include "resource.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(msi);
@@ -288,8 +290,6 @@ static void free_package_structures( MSIPACKAGE *package )
         MSIBINARY *binary = LIST_ENTRY( item, MSIBINARY, entry );
 
         list_remove( &binary->entry );
-        if (binary->module)
-            FreeLibrary( binary->module );
         if (!DeleteFileW( binary->tmpfile ))
             ERR("failed to delete %s (%u)\n", debugstr_w(binary->tmpfile), GetLastError());
         msi_free( binary->source );
@@ -319,7 +319,6 @@ static void free_package_structures( MSIPACKAGE *package )
         msi_free_patchinfo( patch );
     }
 
-    msi_free( package->BaseURL );
     msi_free( package->PackagePath );
     msi_free( package->ProductCode );
     msi_free( package->ActionFormat );
@@ -663,7 +662,7 @@ static VOID set_installer_properties(MSIPACKAGE *package)
     WCHAR *ptr;
     OSVERSIONINFOEXW OSVersion;
     MEMORYSTATUSEX msex;
-    DWORD verval, len;
+    DWORD verval, len, type;
     WCHAR pth[MAX_PATH], verstr[11], bufstr[22];
     HDC dc;
     HKEY hkey;
@@ -709,6 +708,10 @@ static VOID set_installer_properties(MSIPACKAGE *package)
     static const WCHAR szSystem64Folder[] = { 'S','y','s','t','e','m','6','4','F','o','l','d','e','r',0 };
     static const WCHAR szCommonFiles64Folder[] = { 'C','o','m','m','o','n','F','i','l','e','s','6','4','F','o','l','d','e','r',0 };
     static const WCHAR szProgramFiles64Folder[] = { 'P','r','o','g','r','a','m','F','i','l','e','s','6','4','F','o','l','d','e','r',0 };
+    static const WCHAR szProgramFilesDir[] = {'P','r','o','g','r','a','m','F','i','l','e','s','D','i','r',0};
+    static const WCHAR szProgramFilesDirx86[] = {'P','r','o','g','r','a','m','F','i','l','e','s','D','i','r',' ','(','x','8','6',')',0};
+    static const WCHAR szCommonFilesDir[] = {'C','o','m','m','o','n','F','i','l','e','s','D','i','r',0};
+    static const WCHAR szCommonFilesDirx86[] = {'C','o','m','m','o','n','F','i','l','e','s','D','i','r',' ','(','x','8','6',')',0};
     static const WCHAR szVersionNT64[] = { 'V','e','r','s','i','o','n','N','T','6','4',0 };
     static const WCHAR szUserInfo[] = {
         'S','O','F','T','W','A','R','E','\\',
@@ -719,6 +722,12 @@ static VOID set_installer_properties(MSIPACKAGE *package)
     static const WCHAR szDefName[] = { 'D','e','f','N','a','m','e',0 };
     static const WCHAR szDefCompany[] = { 'D','e','f','C','o','m','p','a','n','y',0 };
     static const WCHAR szCurrentVersion[] = {
+        'S','O','F','T','W','A','R','E','\\',
+        'M','i','c','r','o','s','o','f','t','\\',
+        'W','i','n','d','o','w','s','\\',
+        'C','u','r','r','e','n','t','V','e','r','s','i','o','n',0
+    };
+    static const WCHAR szCurrentVersionNT[] = {
         'S','O','F','T','W','A','R','E','\\',
         'M','i','c','r','o','s','o','f','t','\\',
         'W','i','n','d','o','w','s',' ','N','T','\\',
@@ -868,6 +877,9 @@ static VOID set_installer_properties(MSIPACKAGE *package)
     len = sprintfW( bufstr, szFormat, MSI_MAJORVERSION * 100 );
     msi_set_property( package->db, szVersionDatabase, bufstr, len );
 
+    RegOpenKeyExW(HKEY_LOCAL_MACHINE, szCurrentVersion, 0,
+        KEY_QUERY_VALUE | KEY_WOW64_64KEY, &hkey);
+
     GetNativeSystemInfo( &sys_info );
     len = sprintfW( bufstr, szIntFormat, sys_info.wProcessorLevel );
     msi_set_property( package->db, szIntel, bufstr, len );
@@ -877,11 +889,13 @@ static VOID set_installer_properties(MSIPACKAGE *package)
         PathAddBackslashW( pth );
         msi_set_property( package->db, szSystemFolder, pth, -1 );
 
-        SHGetFolderPathW( NULL, CSIDL_PROGRAM_FILES, NULL, 0, pth );
+        len = MAX_PATH;
+        RegQueryValueExW(hkey, szProgramFilesDir, 0, &type, (BYTE *)pth, &len);
         PathAddBackslashW( pth );
         msi_set_property( package->db, szProgramFilesFolder, pth, -1 );
 
-        SHGetFolderPathW( NULL, CSIDL_PROGRAM_FILES_COMMON, NULL, 0, pth );
+        len = MAX_PATH;
+        RegQueryValueExW(hkey, szCommonFilesDir, 0, &type, (BYTE *)pth, &len);
         PathAddBackslashW( pth );
         msi_set_property( package->db, szCommonFilesFolder, pth, -1 );
     }
@@ -899,23 +913,29 @@ static VOID set_installer_properties(MSIPACKAGE *package)
         PathAddBackslashW( pth );
         msi_set_property( package->db, szSystemFolder, pth, -1 );
 
-        SHGetFolderPathW( NULL, CSIDL_PROGRAM_FILES, NULL, 0, pth );
+        len = MAX_PATH;
+        RegQueryValueExW(hkey, szProgramFilesDir, 0, &type, (BYTE *)pth, &len);
         PathAddBackslashW( pth );
         msi_set_property( package->db, szProgramFiles64Folder, pth, -1 );
 
-        SHGetFolderPathW( NULL, CSIDL_PROGRAM_FILESX86, NULL, 0, pth );
+        len = MAX_PATH;
+        RegQueryValueExW(hkey, szProgramFilesDirx86, 0, &type, (BYTE *)pth, &len);
         PathAddBackslashW( pth );
         msi_set_property( package->db, szProgramFilesFolder, pth, -1 );
 
-        SHGetFolderPathW( NULL, CSIDL_PROGRAM_FILES_COMMON, NULL, 0, pth );
+        len = MAX_PATH;
+        RegQueryValueExW(hkey, szCommonFilesDir, 0, &type, (BYTE *)pth, &len);
         PathAddBackslashW( pth );
         msi_set_property( package->db, szCommonFiles64Folder, pth, -1 );
 
-        SHGetFolderPathW( NULL, CSIDL_PROGRAM_FILES_COMMONX86, NULL, 0, pth );
+        len = MAX_PATH;
+        RegQueryValueExW(hkey, szCommonFilesDirx86, 0, &type, (BYTE *)pth, &len);
         PathAddBackslashW( pth );
         msi_set_property( package->db, szCommonFilesFolder, pth, -1 );
     }
 
+    RegCloseKey(hkey);
+
     /* Screen properties. */
     dc = GetDC(0);
     len = sprintfW( bufstr, szIntFormat, GetDeviceCaps(dc, HORZRES) );
@@ -942,7 +962,7 @@ static VOID set_installer_properties(MSIPACKAGE *package)
         CloseHandle( hkey );
     }
     if ((!username || !companyname) &&
-        RegOpenKeyW( HKEY_LOCAL_MACHINE, szCurrentVersion, &hkey ) == ERROR_SUCCESS)
+        RegOpenKeyW( HKEY_LOCAL_MACHINE, szCurrentVersionNT, &hkey ) == ERROR_SUCCESS)
     {
         if (!username &&
             (username = msi_reg_get_val_str( hkey, szRegisteredUser )))
@@ -1054,7 +1074,7 @@ void msi_adjust_privilege_properties( MSIPACKAGE *package )
     msi_set_property( package->db, szAdminUser, szOne, -1 );
 }
 
-MSIPACKAGE *MSI_CreatePackage( MSIDATABASE *db, LPCWSTR base_url )
+MSIPACKAGE *MSI_CreatePackage( MSIDATABASE *db )
 {
     static const WCHAR fmtW[] = {'%','u',0};
     MSIPACKAGE *package;
@@ -1075,7 +1095,6 @@ MSIPACKAGE *MSI_CreatePackage( MSIDATABASE *db, LPCWSTR base_url )
         package->LastActionResult = MSI_NULL_INTEGER;
         package->WordCount = 0;
         package->PackagePath = strdupW( db->path );
-        package->BaseURL = strdupW( base_url );
 
         create_temp_property_table( package );
         msi_clone_properties( package->db );
@@ -1445,15 +1464,13 @@ UINT MSI_OpenPackageW(LPCWSTR szPackage, MSIPACKAGE **pPackage)
     MSIPACKAGE *package;
     MSIHANDLE handle;
     MSIRECORD *data_row, *info_row;
-    LPWSTR ptr, base_url = NULL;
     UINT r;
     WCHAR localfile[MAX_PATH], cachefile[MAX_PATH];
     LPCWSTR file = szPackage;
     DWORD index = 0;
     MSISUMMARYINFO *si;
     BOOL delete_on_close = FALSE;
-    LPWSTR productname;
-    WCHAR *info_template;
+    WCHAR *info_template, *productname;
 
     TRACE("%s %p\n", debugstr_w(szPackage), pPackage);
 
@@ -1466,13 +1483,11 @@ UINT MSI_OpenPackageW(LPCWSTR szPackage, MSIPACKAGE **pPackage)
         db = msihandle2msiinfo( handle, MSIHANDLETYPE_DATABASE );
         if( !db )
         {
-            IWineMsiRemoteDatabase *remote_database;
+            MSIHANDLE remote;
 
-            remote_database = (IWineMsiRemoteDatabase *)msi_get_remote( handle );
-            if ( !remote_database )
+            if (!(remote = msi_get_remote(handle)))
                 return ERROR_INVALID_HANDLE;
 
-            IWineMsiRemoteDatabase_Release( remote_database );
             WARN("MsiOpenPackage not allowed during a custom action!\n");
 
             return ERROR_FUNCTION_FAILED;
@@ -1487,30 +1502,19 @@ UINT MSI_OpenPackageW(LPCWSTR szPackage, MSIPACKAGE **pPackage)
                 return r;
 
             file = cachefile;
-
-            base_url = strdupW( szPackage );
-            if (!base_url)
-                return ERROR_OUTOFMEMORY;
-
-            ptr = strrchrW( base_url, '/' );
-            if (ptr) *(ptr + 1) = '\0';
         }
         r = get_local_package( file, localfile );
         if (r != ERROR_SUCCESS || GetFileAttributesW( localfile ) == INVALID_FILE_ATTRIBUTES)
         {
             r = msi_create_empty_local_file( localfile, dotmsi );
             if (r != ERROR_SUCCESS)
-            {
-                msi_free ( base_url );
                 return r;
-            }
 
             if (!CopyFileW( file, localfile, FALSE ))
             {
                 r = GetLastError();
                 WARN("unable to copy package %s to %s (%u)\n", debugstr_w(file), debugstr_w(localfile), r);
                 DeleteFileW( localfile );
-                msi_free ( base_url );
                 return r;
             }
             delete_on_close = TRUE;
@@ -1518,13 +1522,9 @@ UINT MSI_OpenPackageW(LPCWSTR szPackage, MSIPACKAGE **pPackage)
         TRACE("opening package %s\n", debugstr_w( localfile ));
         r = MSI_OpenDatabaseW( localfile, MSIDBOPEN_TRANSACT, &db );
         if (r != ERROR_SUCCESS)
-        {
-            msi_free ( base_url );
             return r;
-        }
     }
-    package = MSI_CreatePackage( db, base_url );
-    msi_free( base_url );
+    package = MSI_CreatePackage( db );
     msiobj_release( &db->hdr );
     if (!package) return ERROR_INSTALL_PACKAGE_INVALID;
     package->localfile = strdupW( localfile );
@@ -1693,8 +1693,7 @@ MSIHANDLE WINAPI MsiGetActiveDatabase(MSIHANDLE hInstall)
 {
     MSIPACKAGE *package;
     MSIHANDLE handle = 0;
-    IUnknown *remote_unk;
-    IWineMsiRemotePackage *remote_package;
+    MSIHANDLE remote;
 
     TRACE("(%d)\n",hInstall);
 
@@ -1704,19 +1703,10 @@ MSIHANDLE WINAPI MsiGetActiveDatabase(MSIHANDLE hInstall)
         handle = alloc_msihandle( &package->db->hdr );
         msiobj_release( &package->hdr );
     }
-    else if ((remote_unk = msi_get_remote(hInstall)))
+    else if ((remote = msi_get_remote(hInstall)))
     {
-        if (IUnknown_QueryInterface(remote_unk, &IID_IWineMsiRemotePackage,
-                                        (LPVOID *)&remote_package) == S_OK)
-        {
-            IWineMsiRemotePackage_GetActiveDatabase(remote_package, &handle);
-            IWineMsiRemotePackage_Release(remote_package);
-        }
-        else
-        {
-            WARN("remote handle %d is not a package\n", hInstall);
-        }
-        IUnknown_Release(remote_unk);
+        handle = remote_GetActiveDatabase(remote);
+        handle = alloc_msi_remote_handle(handle);
     }
 
     return handle;
@@ -2064,42 +2054,27 @@ INT WINAPI MsiProcessMessage( MSIHANDLE hInstall, INSTALLMESSAGE eMessageType,
         MsiRecordGetInteger(hRecord, 1) != 2)
         return -1;
 
+    record = msihandle2msiinfo(hRecord, MSIHANDLETYPE_RECORD);
+    if (!record)
+        return ERROR_INVALID_HANDLE;
+
     package = msihandle2msiinfo( hInstall, MSIHANDLETYPE_PACKAGE );
     if( !package )
     {
-        HRESULT hr;
-        IWineMsiRemotePackage *remote_package;
+        MSIHANDLE remote;
 
-        remote_package = (IWineMsiRemotePackage *)msi_get_remote( hInstall );
-        if (!remote_package)
+        if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        hr = IWineMsiRemotePackage_ProcessMessage( remote_package, eMessageType, hRecord );
+        ret = remote_ProcessMessage(remote, eMessageType, (struct wire_record *)&record->count);
 
-        IWineMsiRemotePackage_Release( remote_package );
-
-        if (FAILED(hr))
-        {
-            if (HRESULT_FACILITY(hr) == FACILITY_WIN32)
-                return HRESULT_CODE(hr);
-
-            return ERROR_FUNCTION_FAILED;
-        }
-
-        return ERROR_SUCCESS;
+        msiobj_release(&record->hdr);
+        return ret;
     }
 
-    record = msihandle2msiinfo( hRecord, MSIHANDLETYPE_RECORD );
-    if( !record )
-        goto out;
-
     ret = MSI_ProcessMessage( package, eMessageType, record );
 
-out:
     msiobj_release( &package->hdr );
-    if( record )
-        msiobj_release( &record->hdr );
-
     return ret;
 }
 
@@ -2216,39 +2191,12 @@ UINT WINAPI MsiSetPropertyW( MSIHANDLE hInstall, LPCWSTR szName, LPCWSTR szValue
     package = msihandle2msiinfo( hInstall, MSIHANDLETYPE_PACKAGE);
     if( !package )
     {
-        HRESULT hr;
-        BSTR name = NULL, value = NULL;
-        IWineMsiRemotePackage *remote_package;
+        MSIHANDLE remote;
 
-        remote_package = (IWineMsiRemotePackage *)msi_get_remote( hInstall );
-        if (!remote_package)
+        if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        name = SysAllocString( szName );
-        value = SysAllocString( szValue );
-        if ((!name && szName) || (!value && szValue))
-        {
-            SysFreeString( name );
-            SysFreeString( value );
-            IWineMsiRemotePackage_Release( remote_package );
-            return ERROR_OUTOFMEMORY;
-        }
-
-        hr = IWineMsiRemotePackage_SetProperty( remote_package, name, value );
-
-        SysFreeString( name );
-        SysFreeString( value );
-        IWineMsiRemotePackage_Release( remote_package );
-
-        if (FAILED(hr))
-        {
-            if (HRESULT_FACILITY(hr) == FACILITY_WIN32)
-                return HRESULT_CODE(hr);
-
-            return ERROR_FUNCTION_FAILED;
-        }
-
-        return ERROR_SUCCESS;
+        return remote_SetProperty(remote, szName, szValue);
     }
 
     ret = msi_set_property( package->db, szName, szValue, -1 );
@@ -2415,57 +2363,34 @@ static UINT MSI_GetProperty( MSIHANDLE handle, LPCWSTR name,
     package = msihandle2msiinfo( handle, MSIHANDLETYPE_PACKAGE );
     if (!package)
     {
-        HRESULT hr;
-        IWineMsiRemotePackage *remote_package;
-        LPWSTR value = NULL;
-        BSTR bname;
+        LPWSTR value = NULL, buffer;
+        MSIHANDLE remote;
 
-        remote_package = (IWineMsiRemotePackage *)msi_get_remote( handle );
-        if (!remote_package)
+        if (!(remote = msi_get_remote(handle)))
             return ERROR_INVALID_HANDLE;
 
-        bname = SysAllocString( name );
-        if (!bname)
+        r = remote_GetProperty(remote, name, &value, &len);
+        if (r != ERROR_SUCCESS)
+            return r;
+
+        /* String might contain embedded nulls.
+         * Native returns the correct size but truncates the string. */
+        buffer = heap_alloc_zero((len + 1) * sizeof(WCHAR));
+        if (!buffer)
         {
-            IWineMsiRemotePackage_Release( remote_package );
+            midl_user_free(value);
             return ERROR_OUTOFMEMORY;
         }
+        strcpyW(buffer, value);
 
-        hr = IWineMsiRemotePackage_GetProperty( remote_package, bname, NULL, &len );
-        if (FAILED(hr))
-            goto done;
-
-        len++;
-        value = msi_alloc(len * sizeof(WCHAR));
-        if (!value)
-        {
-            r = ERROR_OUTOFMEMORY;
-            goto done;
-        }
-
-        hr = IWineMsiRemotePackage_GetProperty( remote_package, bname, value, &len );
-        if (FAILED(hr))
-            goto done;
-
-        r = msi_strcpy_to_awstring( value, len, szValueBuf, pchValueBuf );
+        r = msi_strcpy_to_awstring(buffer, len, szValueBuf, pchValueBuf);
 
         /* Bug required by Adobe installers */
-        if (!szValueBuf->unicode && !szValueBuf->str.a)
+        if (pchValueBuf && !szValueBuf->unicode && !szValueBuf->str.a)
             *pchValueBuf *= sizeof(WCHAR);
 
-done:
-        IWineMsiRemotePackage_Release(remote_package);
-        SysFreeString(bname);
-        msi_free(value);
-
-        if (FAILED(hr))
-        {
-            if (HRESULT_FACILITY(hr) == FACILITY_WIN32)
-                return HRESULT_CODE(hr);
-
-            return ERROR_FUNCTION_FAILED;
-        }
-
+        heap_free(buffer);
+        midl_user_free(value);
         return r;
     }
 
@@ -2515,286 +2440,179 @@ UINT WINAPI MsiGetPropertyW( MSIHANDLE hInstall, LPCWSTR szName,
     return MSI_GetProperty( hInstall, szName, &val, pchValueBuf );
 }
 
-typedef struct _msi_remote_package_impl {
-    IWineMsiRemotePackage IWineMsiRemotePackage_iface;
-    MSIHANDLE package;
-    LONG refs;
-} msi_remote_package_impl;
-
-static inline msi_remote_package_impl *impl_from_IWineMsiRemotePackage( IWineMsiRemotePackage *iface )
+MSIHANDLE __cdecl s_remote_GetActiveDatabase(MSIHANDLE hinst)
 {
-    return CONTAINING_RECORD(iface, msi_remote_package_impl, IWineMsiRemotePackage_iface);
+    return MsiGetActiveDatabase(hinst);
 }
 
-static HRESULT WINAPI mrp_QueryInterface( IWineMsiRemotePackage *iface,
-                REFIID riid,LPVOID *ppobj)
+UINT __cdecl s_remote_GetProperty(MSIHANDLE hinst, LPCWSTR property, LPWSTR *value, DWORD *size)
 {
-    if( IsEqualCLSID( riid, &IID_IUnknown ) ||
-        IsEqualCLSID( riid, &IID_IWineMsiRemotePackage ) )
+    WCHAR empty[1];
+    UINT r;
+
+    *size = 0;
+    r = MsiGetPropertyW(hinst, property, empty, size);
+    if (r == ERROR_MORE_DATA)
     {
-        IWineMsiRemotePackage_AddRef( iface );
-        *ppobj = iface;
-        return S_OK;
-    }
-
-    return E_NOINTERFACE;
-}
-
-static ULONG WINAPI mrp_AddRef( IWineMsiRemotePackage *iface )
-{
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-
-    return InterlockedIncrement( &This->refs );
-}
-
-static ULONG WINAPI mrp_Release( IWineMsiRemotePackage *iface )
-{
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    ULONG r;
-
-    r = InterlockedDecrement( &This->refs );
-    if (r == 0)
-    {
-        MsiCloseHandle( This->package );
-        msi_free( This );
+        ++*size;
+        *value = midl_user_allocate(*size * sizeof(WCHAR));
+        if (!*value)
+            return ERROR_OUTOFMEMORY;
+        r = MsiGetPropertyW(hinst, property, *value, size);
     }
     return r;
 }
 
-static HRESULT WINAPI mrp_SetMsiHandle( IWineMsiRemotePackage *iface, MSIHANDLE handle )
+UINT __cdecl s_remote_SetProperty(MSIHANDLE hinst, LPCWSTR property, LPCWSTR value)
 {
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    This->package = handle;
-    return S_OK;
+    return MsiSetPropertyW(hinst, property, value);
 }
 
-static HRESULT WINAPI mrp_GetActiveDatabase( IWineMsiRemotePackage *iface, MSIHANDLE *handle )
+int __cdecl s_remote_ProcessMessage(MSIHANDLE hinst, INSTALLMESSAGE message, struct wire_record *remote_rec)
 {
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    IWineMsiRemoteDatabase *rdb = NULL;
-    HRESULT hr;
-    MSIHANDLE hdb;
+    MSIHANDLE rec;
+    int ret;
+    UINT r;
 
-    hr = create_msi_remote_database( NULL, (LPVOID *)&rdb );
-    if (FAILED(hr) || !rdb)
-    {
-        ERR("Failed to create remote database\n");
-        return hr;
-    }
+    if ((r = unmarshal_record(remote_rec, &rec)))
+        return r;
 
-    hdb = MsiGetActiveDatabase(This->package);
+    ret = MsiProcessMessage(hinst, message, rec);
 
-    hr = IWineMsiRemoteDatabase_SetMsiHandle( rdb, hdb );
-    if (FAILED(hr))
-    {
-        ERR("Failed to set the database handle\n");
-        return hr;
-    }
-
-    *handle = alloc_msi_remote_handle( (IUnknown *)rdb );
-    return S_OK;
+    MsiCloseHandle(rec);
+    return ret;
 }
 
-static HRESULT WINAPI mrp_GetProperty( IWineMsiRemotePackage *iface, BSTR property, BSTR value, DWORD *size )
+UINT __cdecl s_remote_DoAction(MSIHANDLE hinst, LPCWSTR action)
 {
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    UINT r = MsiGetPropertyW(This->package, property, value, size);
-    if (r != ERROR_SUCCESS) return HRESULT_FROM_WIN32(r);
-    return S_OK;
+    return MsiDoActionW(hinst, action);
 }
 
-static HRESULT WINAPI mrp_SetProperty( IWineMsiRemotePackage *iface, BSTR property, BSTR value )
+UINT __cdecl s_remote_Sequence(MSIHANDLE hinst, LPCWSTR table, int sequence)
 {
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    UINT r = MsiSetPropertyW(This->package, property, value);
-    return HRESULT_FROM_WIN32(r);
+    return MsiSequenceW(hinst, table, sequence);
 }
 
-static HRESULT WINAPI mrp_ProcessMessage( IWineMsiRemotePackage *iface, INSTALLMESSAGE message, MSIHANDLE record )
-{
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    UINT r = MsiProcessMessage(This->package, message, record);
-    return HRESULT_FROM_WIN32(r);
-}
-
-static HRESULT WINAPI mrp_DoAction( IWineMsiRemotePackage *iface, BSTR action )
-{
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    UINT r = MsiDoActionW(This->package, action);
-    return HRESULT_FROM_WIN32(r);
-}
-
-static HRESULT WINAPI mrp_Sequence( IWineMsiRemotePackage *iface, BSTR table, int sequence )
-{
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    UINT r = MsiSequenceW(This->package, table, sequence);
-    return HRESULT_FROM_WIN32(r);
-}
-
-static HRESULT WINAPI mrp_GetTargetPath( IWineMsiRemotePackage *iface, BSTR folder, BSTR value, DWORD *size )
-{
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    UINT r = MsiGetTargetPathW(This->package, folder, value, size);
-    return HRESULT_FROM_WIN32(r);
-}
-
-static HRESULT WINAPI mrp_SetTargetPath( IWineMsiRemotePackage *iface, BSTR folder, BSTR value)
-{
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    UINT r = MsiSetTargetPathW(This->package, folder, value);
-    return HRESULT_FROM_WIN32(r);
-}
-
-static HRESULT WINAPI mrp_GetSourcePath( IWineMsiRemotePackage *iface, BSTR folder, BSTR value, DWORD *size )
-{
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    UINT r = MsiGetSourcePathW(This->package, folder, value, size);
-    return HRESULT_FROM_WIN32(r);
-}
-
-static HRESULT WINAPI mrp_GetMode( IWineMsiRemotePackage *iface, MSIRUNMODE mode, BOOL *ret )
-{
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    *ret = MsiGetMode(This->package, mode);
-    return S_OK;
-}
-
-static HRESULT WINAPI mrp_SetMode( IWineMsiRemotePackage *iface, MSIRUNMODE mode, BOOL state )
-{
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    UINT r = MsiSetMode(This->package, mode, state);
-    return HRESULT_FROM_WIN32(r);
-}
-
-static HRESULT WINAPI mrp_GetFeatureState( IWineMsiRemotePackage *iface, BSTR feature,
-                                    INSTALLSTATE *installed, INSTALLSTATE *action )
-{
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    UINT r = MsiGetFeatureStateW(This->package, feature, installed, action);
-    return HRESULT_FROM_WIN32(r);
-}
-
-static HRESULT WINAPI mrp_SetFeatureState( IWineMsiRemotePackage *iface, BSTR feature, INSTALLSTATE state )
-{
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    UINT r = MsiSetFeatureStateW(This->package, feature, state);
-    return HRESULT_FROM_WIN32(r);
-}
-
-static HRESULT WINAPI mrp_GetComponentState( IWineMsiRemotePackage *iface, BSTR component,
-                                      INSTALLSTATE *installed, INSTALLSTATE *action )
-{
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    UINT r = MsiGetComponentStateW(This->package, component, installed, action);
-    return HRESULT_FROM_WIN32(r);
-}
-
-static HRESULT WINAPI mrp_SetComponentState( IWineMsiRemotePackage *iface, BSTR component, INSTALLSTATE state )
-{
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    UINT r = MsiSetComponentStateW(This->package, component, state);
-    return HRESULT_FROM_WIN32(r);
-}
-
-static HRESULT WINAPI mrp_GetLanguage( IWineMsiRemotePackage *iface, LANGID *language )
-{
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    *language = MsiGetLanguage(This->package);
-    return S_OK;
-}
-
-static HRESULT WINAPI mrp_SetInstallLevel( IWineMsiRemotePackage *iface, int level )
-{
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    UINT r = MsiSetInstallLevel(This->package, level);
-    return HRESULT_FROM_WIN32(r);
-}
-
-static HRESULT WINAPI mrp_FormatRecord( IWineMsiRemotePackage *iface, MSIHANDLE record,
-                                        BSTR *value)
+UINT __cdecl s_remote_GetTargetPath(MSIHANDLE hinst, LPCWSTR folder, LPWSTR *value)
 {
+    WCHAR empty[1];
     DWORD size = 0;
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    UINT r = MsiFormatRecordW(This->package, record, NULL, &size);
-    if (r == ERROR_SUCCESS)
+    UINT r;
+
+    r = MsiGetTargetPathW(hinst, folder, empty, &size);
+    if (r == ERROR_MORE_DATA)
     {
-        *value = SysAllocStringLen(NULL, size);
+        *value = midl_user_allocate(++size * sizeof(WCHAR));
         if (!*value)
-            return E_OUTOFMEMORY;
-        size++;
-        r = MsiFormatRecordW(This->package, record, *value, &size);
+            return ERROR_OUTOFMEMORY;
+        r = MsiGetTargetPathW(hinst, folder, *value, &size);
     }
-    return HRESULT_FROM_WIN32(r);
+    return r;
 }
 
-static HRESULT WINAPI mrp_EvaluateCondition( IWineMsiRemotePackage *iface, BSTR condition )
+UINT __cdecl s_remote_SetTargetPath(MSIHANDLE hinst, LPCWSTR folder, LPCWSTR value)
 {
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    UINT r = MsiEvaluateConditionW(This->package, condition);
-    return HRESULT_FROM_WIN32(r);
+    return MsiSetTargetPathW(hinst, folder, value);
 }
 
-static HRESULT WINAPI mrp_GetFeatureCost( IWineMsiRemotePackage *iface, BSTR feature,
-                                          INT cost_tree, INSTALLSTATE state, INT *cost )
+UINT __cdecl s_remote_GetSourcePath(MSIHANDLE hinst, LPCWSTR folder, LPWSTR *value)
 {
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    UINT r = MsiGetFeatureCostW(This->package, feature, cost_tree, state, cost);
-    return HRESULT_FROM_WIN32(r);
+    WCHAR empty[1];
+    DWORD size = 1;
+    UINT r;
+
+    r = MsiGetSourcePathW(hinst, folder, empty, &size);
+    if (r == ERROR_MORE_DATA)
+    {
+        *value = midl_user_allocate(++size * sizeof(WCHAR));
+        if (!*value)
+            return ERROR_OUTOFMEMORY;
+        r = MsiGetSourcePathW(hinst, folder, *value, &size);
+    }
+    return r;
 }
 
-static HRESULT WINAPI mrp_EnumComponentCosts( IWineMsiRemotePackage *iface, BSTR component,
-                                              DWORD index, INSTALLSTATE state, BSTR drive,
-                                              DWORD *buflen, INT *cost, INT *temp )
+BOOL __cdecl s_remote_GetMode(MSIHANDLE hinst, MSIRUNMODE mode)
 {
-    msi_remote_package_impl* This = impl_from_IWineMsiRemotePackage( iface );
-    UINT r = MsiEnumComponentCostsW(This->package, component, index, state, drive, buflen, cost, temp);
-    return HRESULT_FROM_WIN32(r);
+    return MsiGetMode(hinst, mode);
 }
 
-static const IWineMsiRemotePackageVtbl msi_remote_package_vtbl =
+UINT __cdecl s_remote_SetMode(MSIHANDLE hinst, MSIRUNMODE mode, BOOL state)
 {
-    mrp_QueryInterface,
-    mrp_AddRef,
-    mrp_Release,
-    mrp_SetMsiHandle,
-    mrp_GetActiveDatabase,
-    mrp_GetProperty,
-    mrp_SetProperty,
-    mrp_ProcessMessage,
-    mrp_DoAction,
-    mrp_Sequence,
-    mrp_GetTargetPath,
-    mrp_SetTargetPath,
-    mrp_GetSourcePath,
-    mrp_GetMode,
-    mrp_SetMode,
-    mrp_GetFeatureState,
-    mrp_SetFeatureState,
-    mrp_GetComponentState,
-    mrp_SetComponentState,
-    mrp_GetLanguage,
-    mrp_SetInstallLevel,
-    mrp_FormatRecord,
-    mrp_EvaluateCondition,
-    mrp_GetFeatureCost,
-    mrp_EnumComponentCosts
-};
+    return MsiSetMode(hinst, mode, state);
+}
 
-HRESULT create_msi_remote_package( IUnknown *pOuter, LPVOID *ppObj )
+UINT __cdecl s_remote_GetFeatureState(MSIHANDLE hinst, LPCWSTR feature,
+                                    INSTALLSTATE *installed, INSTALLSTATE *action)
 {
-    msi_remote_package_impl* This;
+    return MsiGetFeatureStateW(hinst, feature, installed, action);
+}
 
-    This = msi_alloc( sizeof *This );
-    if (!This)
-        return E_OUTOFMEMORY;
+UINT __cdecl s_remote_SetFeatureState(MSIHANDLE hinst, LPCWSTR feature, INSTALLSTATE state)
+{
+    return MsiSetFeatureStateW(hinst, feature, state);
+}
 
-    This->IWineMsiRemotePackage_iface.lpVtbl = &msi_remote_package_vtbl;
-    This->package = 0;
-    This->refs = 1;
+UINT __cdecl s_remote_GetComponentState(MSIHANDLE hinst, LPCWSTR component,
+                                      INSTALLSTATE *installed, INSTALLSTATE *action)
+{
+    return MsiGetComponentStateW(hinst, component, installed, action);
+}
 
-    *ppObj = &This->IWineMsiRemotePackage_iface;
+UINT __cdecl s_remote_SetComponentState(MSIHANDLE hinst, LPCWSTR component, INSTALLSTATE state)
+{
+    return MsiSetComponentStateW(hinst, component, state);
+}
 
-    return S_OK;
+LANGID __cdecl s_remote_GetLanguage(MSIHANDLE hinst)
+{
+    return MsiGetLanguage(hinst);
+}
+
+UINT __cdecl s_remote_SetInstallLevel(MSIHANDLE hinst, int level)
+{
+    return MsiSetInstallLevel(hinst, level);
+}
+
+UINT __cdecl s_remote_FormatRecord(MSIHANDLE hinst, struct wire_record *remote_rec, LPWSTR *value)
+{
+    WCHAR empty[1];
+    DWORD size = 0;
+    MSIHANDLE rec;
+    UINT r;
+
+    if ((r = unmarshal_record(remote_rec, &rec)))
+        return r;
+
+    r = MsiFormatRecordW(hinst, rec, empty, &size);
+    if (r == ERROR_MORE_DATA)
+    {
+        *value = midl_user_allocate(++size * sizeof(WCHAR));
+        if (!*value)
+            return ERROR_OUTOFMEMORY;
+        r = MsiFormatRecordW(hinst, rec, *value, &size);
+    }
+
+    MsiCloseHandle(rec);
+    return r;
+}
+
+MSICONDITION __cdecl s_remote_EvaluateCondition(MSIHANDLE hinst, LPCWSTR condition)
+{
+    return MsiEvaluateConditionW(hinst, condition);
+}
+
+UINT __cdecl s_remote_GetFeatureCost(MSIHANDLE hinst, LPCWSTR feature,
+    MSICOSTTREE cost_tree, INSTALLSTATE state, INT *cost)
+{
+    return MsiGetFeatureCostW(hinst, feature, cost_tree, state, cost);
+}
+
+UINT __cdecl s_remote_EnumComponentCosts(MSIHANDLE hinst, LPCWSTR component,
+    DWORD index, INSTALLSTATE state, LPWSTR drive, INT *cost, INT *temp)
+{
+    DWORD size = 3;
+    return MsiEnumComponentCostsW(hinst, component, index, state, drive, &size, cost, temp);
 }
 
 UINT msi_package_add_info(MSIPACKAGE *package, DWORD context, DWORD options,
