@@ -862,8 +862,15 @@ static void test_DragQueryFile(void)
 
 static void test_SHCreateSessionKey(void)
 {
+    static const WCHAR session_format[] = {
+                'S','o','f','t','w','a','r','e','\\','M','i','c','r','o','s','o','f','t','\\',
+                'W','i','n','d','o','w','s','\\','C','u','r','r','e','n','t','V','e','r','s','i','o','n','\\',
+                'E','x','p','l','o','r','e','r','\\','S','e','s','s','i','o','n','I','n','f','o','\\','%','u',0};
     HKEY hkey, hkey2;
     HRESULT hr;
+    DWORD session;
+    WCHAR sessionW[(sizeof(session_format)/sizeof(WCHAR)) + 16];
+    LONG ret;
 
     if (!pSHCreateSessionKey)
     {
@@ -876,18 +883,28 @@ static void test_SHCreateSessionKey(void)
 
     hkey = (HKEY)0xdeadbeef;
     hr = pSHCreateSessionKey(0, &hkey);
-    todo_wine ok(hr == E_ACCESSDENIED, "got 0x%08x\n", hr);
+    ok(hr == E_ACCESSDENIED, "got 0x%08x\n", hr);
     ok(hkey == NULL, "got %p\n", hkey);
 
     hr = pSHCreateSessionKey(KEY_READ, &hkey);
-    todo_wine ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
 
     hr = pSHCreateSessionKey(KEY_READ, &hkey2);
-    todo_wine ok(hr == S_OK, "got 0x%08x\n", hr);
-    todo_wine ok(hkey != hkey2, "got %p, %p\n", hkey, hkey2);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(hkey != hkey2, "got %p, %p\n", hkey, hkey2);
 
     RegCloseKey(hkey);
     RegCloseKey(hkey2);
+
+    /* check the registry */
+    ProcessIdToSessionId( GetCurrentProcessId(), &session);
+    if (session)
+    {
+        wsprintfW(sessionW, session_format, session);
+        ret = RegOpenKeyW(HKEY_CURRENT_USER, sessionW, &hkey);
+        ok(!ret, "key not found\n");
+        RegCloseKey(hkey);
+    }
 }
 
 static void test_dragdrophelper(void)
